@@ -44,6 +44,7 @@ CPU_GCC_TRIPLE_RISCV32 = (
     "riscv64-elf",
     "riscv32-elf",
     "riscv-none-embed",
+    "riscv-none-elf",
     "riscv64-linux",
     "riscv64-linux-gnu",
     "riscv-sifive-elf",
@@ -77,6 +78,7 @@ from litex.soc.cores.cpu.picorv32 import PicoRV32
 from litex.soc.cores.cpu.minerva import Minerva
 from litex.soc.cores.cpu.vexriscv import VexRiscv
 from litex.soc.cores.cpu.vexriscv_smp import VexRiscvSMP
+from litex.soc.cores.cpu.ibex import Ibex
 from litex.soc.cores.cpu.cv32e40p import CV32E40P
 
 # RISC-V (64-bit)
@@ -109,6 +111,7 @@ CPUS = {
     "minerva"     : Minerva,
     "vexriscv"    : VexRiscv,
     "vexriscv_smp": VexRiscvSMP,
+    "ibex"        : Ibex,
     "cv32e40p"    : CV32E40P,
 
     # RISC-V (64-bit)
