@@ -16,8 +16,14 @@ from litex.soc.interconnect.csr import *
 from litex.gen.common import reverse_bytes
 from litex.soc.cores.cpu import CPU
 
+class Open(Signal): pass
+
+# Variants -----------------------------------------------------------------------------------------
+
 CPU_VARIANTS = ["standard", "standard+ghdl", "standard+irq", "standard+ghdl+irq"]
 
+# Microwatt ----------------------------------------------------------------------------------------
+
 class Microwatt(CPU):
     name                 = "microwatt"
     human_name           = "Microwatt"
@@ -27,16 +33,19 @@ class Microwatt(CPU):
     gcc_triple           = ("powerpc64le-linux", "powerpc64le-linux-gnu")
     linker_output_format = "elf64-powerpcle"
     nop                  = "nop"
-    io_regions           = {0xc0000000: 0x10000000} # origin, length
+    io_regions           = {0xc0000000: 0x10000000} # Origin, Length.
 
+    # Memory Mapping.
     @property
     def mem_map(self):
         return {
-            "csr":      0xc0000000,
-            "xicsicp":  0xc3ff0000,
-            "xicsics":  0xc3ff1000
+            # Keep the lower 128MBs for SoC IOs auto-allocation.
+            "csr":      0xc8000000,
+            "xicsicp":  0xcbff0000,
+            "xicsics":  0xcbff1000,
         }
 
+    # GCC Flags.
     @property
     def gcc_flags(self):
         flags  = "-m64 "
@@ -57,58 +66,58 @@ class Microwatt(CPU):
         self.platform     = platform
         self.variant      = variant
         self.reset        = Signal()
-        self.wb_insn      = wb_insn = wishbone.Interface(data_width=64, adr_width=29)
-        self.wb_data      = wb_data = wishbone.Interface(data_width=64, adr_width=29)
-        self.periph_buses = [wb_insn, wb_data]
-        self.memory_buses = []
+        self.ibus         = ibus = wishbone.Interface(data_width=64, adr_width=29)
+        self.dbus         = dbus = wishbone.Interface(data_width=64, adr_width=29)
+        self.periph_buses = [ibus, dbus] # Peripheral buses (Connected to main SoC's bus).
+        self.memory_buses = []           # Memory buses (Connected directly to LiteDRAM).
         if "irq" in variant:
-            self.interrupt    = Signal(16)
+            self.interrupt = Signal(16)
         self.core_ext_irq = Signal()
 
         # # #
 
         self.cpu_params = dict(
-            # Clock / Reset
-            i_clk                 = ClockSignal(),
-            i_rst                 = ResetSignal() | self.reset,
+            # Clk / Rst.
+            i_clk = ClockSignal(),
+            i_rst = ResetSignal() | self.reset,
 
-            # Wishbone instruction bus
-            i_wishbone_insn_dat_r = wb_insn.dat_r,
-            i_wishbone_insn_ack   = wb_insn.ack,
-            i_wishbone_insn_stall = wb_insn.cyc & ~wb_insn.ack, # No burst support
+            # IBus.
+            i_wishbone_insn_dat_r = ibus.dat_r,
+            i_wishbone_insn_ack   = ibus.ack,
+            i_wishbone_insn_stall = ibus.cyc & ~ibus.ack, # No burst support
 
-            o_wishbone_insn_adr   = Cat(Signal(3), wb_insn.adr),
-            o_wishbone_insn_dat_w = wb_insn.dat_w,
-            o_wishbone_insn_cyc   = wb_insn.cyc,
-            o_wishbone_insn_stb   = wb_insn.stb,
-            o_wishbone_insn_sel   = wb_insn.sel,
-            o_wishbone_insn_we    = wb_insn.we,
+            o_wishbone_insn_adr   = Cat(Signal(3), ibus.adr),
+            o_wishbone_insn_dat_w = ibus.dat_w,
+            o_wishbone_insn_cyc   = ibus.cyc,
+            o_wishbone_insn_stb   = ibus.stb,
+            o_wishbone_insn_sel   = ibus.sel,
+            o_wishbone_insn_we    = ibus.we,
 
-            # Wishbone data bus
-            i_wishbone_data_dat_r = wb_data.dat_r,
-            i_wishbone_data_ack   = wb_data.ack,
-            i_wishbone_data_stall = wb_data.cyc & ~wb_data.ack, # No burst support
+            # DBus.
+            i_wishbone_data_dat_r = dbus.dat_r,
+            i_wishbone_data_ack   = dbus.ack,
+            i_wishbone_data_stall = dbus.cyc & ~dbus.ack, # No burst support
 
-            o_wishbone_data_adr   = Cat(Signal(3), wb_data.adr),
-            o_wishbone_data_dat_w = wb_data.dat_w,
-            o_wishbone_data_cyc   = wb_data.cyc,
-            o_wishbone_data_stb   = wb_data.stb,
-            o_wishbone_data_sel   = wb_data.sel,
-            o_wishbone_data_we    = wb_data.we,
+            o_wishbone_data_adr   = Cat(Signal(3), dbus.adr),
+            o_wishbone_data_dat_w = dbus.dat_w,
+            o_wishbone_data_cyc   = dbus.cyc,
+            o_wishbone_data_stb   = dbus.stb,
+            o_wishbone_data_sel   = dbus.sel,
+            o_wishbone_data_we    = dbus.we,
 
-            # Debug bus
-            i_dmi_addr            = 0,
-            i_dmi_din             = 0,
-            #o_dmi_dout           =,
-            i_dmi_req             = 0,
-            i_dmi_wr              = 0,
-            #o_dmi_ack            =,
+            # Debug.
+            i_dmi_addr = 0,
+            i_dmi_din  = 0,
+            o_dmi_dout = Open(),
+            i_dmi_req  = 0,
+            i_dmi_wr   = 0,
+            o_dmi_ack  = Open(),
 
-            # Interrupt controller
-            i_core_ext_irq        = self.core_ext_irq,
+            # IRQ.
+            i_core_ext_irq = self.core_ext_irq,
         )
 
-        # add vhdl sources
+        # Add VHDL sources.
         self.add_sources(platform, use_ghdl_yosys_plugin="ghdl" in self.variant)
 
     def set_reset_address(self, reset_address):
@@ -117,38 +126,38 @@ class Microwatt(CPU):
         assert reset_address == 0x00000000
 
     def add_soc_components(self, soc, soc_region_cls):
-        self.submodules.xics = XICSSlave(
-            platform     = self.platform,
-            variant      = self.variant,
-            core_irq_out = self.core_ext_irq,
-            int_level_in = self.interrupt,
-            endianness   = self.endianness
-        )
-        xicsicp_region = soc_region_cls(origin=soc.mem_map.get("xicsicp"), size=4096, cached=False)
-        xicsics_region = soc_region_cls(origin=soc.mem_map.get("xicsics"), size=4096, cached=False)
-        soc.bus.add_slave(name="xicsicp", slave=self.xics.icp_bus, region=xicsicp_region)
-        soc.bus.add_slave(name="xicsics", slave=self.xics.ics_bus, region=xicsics_region)
+        if "irq" in self.variant:
+            self.submodules.xics = XICSSlave(
+                platform     = self.platform,
+                variant      = self.variant,
+                core_irq_out = self.core_ext_irq,
+                int_level_in = self.interrupt,
+            )
+            xicsicp_region = soc_region_cls(origin=soc.mem_map.get("xicsicp"), size=4096, cached=False)
+            xicsics_region = soc_region_cls(origin=soc.mem_map.get("xicsics"), size=4096, cached=False)
+            soc.bus.add_slave(name="xicsicp", slave=self.xics.icp_bus, region=xicsicp_region)
+            soc.bus.add_slave(name="xicsics", slave=self.xics.ics_bus, region=xicsics_region)
 
     @staticmethod
     def add_sources(platform, use_ghdl_yosys_plugin=False):
         sources = [
-            # Common / Types / Helpers
+            # Common / Types / Helpers.
             "decode_types.vhdl",
             "wishbone_types.vhdl",
             "utils.vhdl",
             "common.vhdl",
             "helpers.vhdl",
 
-            # Fetch
+            # Fetch.
             "fetch1.vhdl",
 
-            # Instruction/Data Cache
+            # Instruction/Data Cache.
             "cache_ram.vhdl",
             "plru.vhdl",
             "dcache.vhdl",
             "icache.vhdl",
 
-            # Decode
+            # Decode.
             "insn_helpers.vhdl",
             "decode1.vhdl",
             "gpr_hazard.vhdl",
@@ -156,37 +165,38 @@ class Microwatt(CPU):
             "control.vhdl",
             "decode2.vhdl",
 
-            # Register/CR File
+            # Register/CR File.
             "register_file.vhdl",
             "crhelpers.vhdl",
             "cr_file.vhdl",
 
-            # Execute
+            # Execute.
             "ppc_fx_insns.vhdl",
             "logical.vhdl",
             "rotator.vhdl",
             "countzero.vhdl",
             "execute1.vhdl",
 
-            # Load/Store
+            # Load/Store.
             "loadstore1.vhdl",
 
-            # Multiply/Divide
+            # Multiply/Divide.
             "multiply.vhdl",
             "divider.vhdl",
 
-            # Writeback
+            # Writeback.
             "writeback.vhdl",
 
-            # MMU
+            # MMU.
             "mmu.vhdl",
 
-            # Core
+            # Core.
             "core_debug.vhdl",
             "core.vhdl",
         ]
         sdir = get_data_mod("cpu", "microwatt").data_location
         cdir = os.path.dirname(__file__)
+        # Convert VHDL to Verilog through GHDL/Yosys.
         if use_ghdl_yosys_plugin:
             from litex.build import tools
             import subprocess
@@ -202,6 +212,7 @@ class Microwatt(CPU):
             if subprocess.call(["yosys", "-q", "-m", "ghdl", os.path.join(cdir, "microwatt.ys")]):
                 raise OSError("Unable to convert Microwatt CPU to verilog, please check your GHDL-Yosys-plugin install")
             platform.add_source(os.path.join(cdir, "microwatt.v"))
+        # Direct use of VHDL sources.
         else:
             platform.add_sources(sdir, *sources)
             platform.add_source(os.path.join(os.path.dirname(__file__), "microwatt_wrapper.vhdl"))
@@ -209,39 +220,30 @@ class Microwatt(CPU):
     def do_finalize(self):
         self.specials += Instance("microwatt_wrapper", **self.cpu_params)
 
+# XICS Slave ---------------------------------------------------------------------------------------
 
 class XICSSlave(Module, AutoCSR):
-    def __init__(self, platform, core_irq_out=Signal(), int_level_in=Signal(16), endianness="big", variant="standard"):
+    def __init__(self, platform, core_irq_out=Signal(), int_level_in=Signal(16), variant="standard"):
         self.variant = variant
 
-        self.icp_bus    = icp_bus    = wishbone.Interface(data_width=32, adr_width=12)
-        self.ics_bus    = ics_bus    = wishbone.Interface(data_width=32, adr_width=12)
+        self.icp_bus = icp_bus = wishbone.Interface(data_width=32, adr_width=12)
+        self.ics_bus = ics_bus = wishbone.Interface(data_width=32, adr_width=12)
 
-        # Bus endianness handlers
-        self.icp_dat_w = Signal(32)
-        self.icp_dat_r = Signal(32)
-        self.comb += self.icp_dat_w.eq(icp_bus.dat_w if endianness == "big" else reverse_bytes(icp_bus.dat_w))
-        self.comb += icp_bus.dat_r.eq(self.icp_dat_r if endianness == "big" else reverse_bytes(self.icp_dat_r))
-        self.ics_dat_w = Signal(32)
-        self.ics_dat_r = Signal(32)
-        self.comb += self.ics_dat_w.eq(ics_bus.dat_w if endianness == "big" else reverse_bytes(ics_bus.dat_w))
-        self.comb += ics_bus.dat_r.eq(self.ics_dat_r if endianness == "big" else reverse_bytes(self.ics_dat_r))
-
-        # XICS signals
+        # XICS signals.
         self.ics_icp_xfer_src = Signal(4)
         self.ics_icp_xfer_pri = Signal(8)
 
         self.icp_params = dict(
-            # Clock / Reset
+            # Clk / Rst.
             i_clk            = ClockSignal(),
             i_rst            = ResetSignal(),
 
-            # Wishbone bus
-            o_wishbone_dat_r = self.icp_dat_r,
+            # Wishbone Bus.
+            o_wishbone_dat_r = icp_bus.dat_r,
             o_wishbone_ack   = icp_bus.ack,
 
             i_wishbone_adr   = icp_bus.adr,
-            i_wishbone_dat_w = self.icp_dat_w,
+            i_wishbone_dat_w = icp_bus.dat_w,
             i_wishbone_cyc   = icp_bus.cyc,
             i_wishbone_stb   = icp_bus.stb,
             i_wishbone_sel   = icp_bus.sel,
@@ -254,16 +256,16 @@ class XICSSlave(Module, AutoCSR):
         )
 
         self.ics_params = dict(
-            # Clock / Reset
+            # Clk / Rst.
             i_clk            = ClockSignal(),
             i_rst            = ResetSignal(),
 
-            # Wishbone bus
-            o_wishbone_dat_r = self.ics_dat_r,
+            # Wishbone Bus.
+            o_wishbone_dat_r = ics_bus.dat_r,
             o_wishbone_ack   = ics_bus.ack,
 
             i_wishbone_adr   = ics_bus.adr,
-            i_wishbone_dat_w = self.ics_dat_w,
+            i_wishbone_dat_w = ics_bus.dat_w,
             i_wishbone_cyc   = ics_bus.cyc,
             i_wishbone_stb   = ics_bus.stb,
             i_wishbone_sel   = ics_bus.sel,
@@ -275,7 +277,7 @@ class XICSSlave(Module, AutoCSR):
             o_icp_out_pri    = self.ics_icp_xfer_pri,
         )
 
-        # add vhdl sources
+        # Add VHDL sources.
         self.add_sources(platform, use_ghdl_yosys_plugin="ghdl" in self.variant)
 
     @staticmethod
