@@ -33,6 +33,10 @@ from litex.soc.doc.module import gather_submodules, ModuleNotDocumented, Documen
 from litex.soc.doc.csr import DocumentedCSRRegion
 from litex.soc.interconnect.csr import _CompoundCSR
 
+# for generating a timestamp in the description field, if none is otherwise given
+import datetime
+import time
+
 # CPU files ----------------------------------------------------------------------------------------
 
 def get_cpu_mak(cpu, compile_software):
@@ -64,6 +68,9 @@ def get_cpu_mak(cpu, compile_software):
         r = None
         if not isinstance(triple, tuple):
             triple = (triple,)
+        override = os.getenv("LITEX_ENV_CC_TRIPLE")
+        if override:
+            triple = (override,) + triple
         p = get_platform()
         for i in range(len(triple)):
             t = triple[i]
@@ -238,7 +245,7 @@ def get_csr_header(regions, constants, csr_base=None, with_access_functions=True
                         size = str(field.size)
                         r += "#define CSR_"+name.upper()+"_"+csr.name.upper()+"_"+field.name.upper()+"_OFFSET "+offset+"\n"
                         r += "#define CSR_"+name.upper()+"_"+csr.name.upper()+"_"+field.name.upper()+"_SIZE "+size+"\n"
-                        if with_access_functions:
+                        if with_access_functions and csr.size <= 32: # FIXME: Implement extract/read functions for csr.size > 32-bit.
                             reg_name = name + "_" + csr.name.lower()
                             field_name = reg_name + "_" + field.name.lower()
                             r += "static inline uint32_t " + field_name + "_extract(uint32_t oldword) {\n"
@@ -394,6 +401,10 @@ def get_csr_svd(soc, vendor="litex", name="soc", description=None):
     svd.append('    <name>{}</name>'.format(name.upper()))
     if description is not None:
         svd.append('    <description><![CDATA[{}]]></description>'.format(reflow(description)))
+    else:
+        fmt = "%Y-%m-%d %H:%M:%S"
+        build_time = datetime.datetime.fromtimestamp(time.time()).strftime(fmt)
+        svd.append('    <description><![CDATA[{}]]></description>'.format(reflow("Litex SoC " + build_time)))
     svd.append('')
     svd.append('    <addressUnitBits>8</addressUnitBits>')
     svd.append('    <width>32</width>')
@@ -460,8 +471,9 @@ def get_csr_svd(soc, vendor="litex", name="soc", description=None):
             svd.append('            </interrupt>')
         svd.append('        </peripheral>')
     svd.append('    </peripherals>')
+    svd.append('    <vendorExtensions>')
+
     if len(soc.mem_regions) > 0:
-        svd.append('    <vendorExtensions>')
         svd.append('        <memoryRegions>')
         for region_name, region in soc.mem_regions.items():
             svd.append('            <memoryRegion>')
@@ -470,7 +482,13 @@ def get_csr_svd(soc, vendor="litex", name="soc", description=None):
             svd.append('                <size>0x{:08X}</size>'.format(region.size))
             svd.append('            </memoryRegion>')
         svd.append('        </memoryRegions>')
-        svd.append('    </vendorExtensions>')
+
+    svd.append('        <constants>')
+    for name, value in soc.constants.items():
+        svd.append('            <constant name="{}" value="{}" />'.format(name, value))
+    svd.append('        </constants>')
+
+    svd.append('    </vendorExtensions>')
     svd.append('</device>')
     return "\n".join(svd)
 
