@@ -49,27 +49,29 @@
 
 static void boot_sequence(void)
 {
-	if(serialboot()) {
+#ifdef CSR_UART_BASE
+	if (serialboot() == 0)
+		return;
+#endif
 #ifdef FLASH_BOOT_ADDRESS
-		flashboot();
+	flashboot();
 #endif
 #ifdef ROM_BOOT_ADDRESS
-		romboot();
+	romboot();
 #endif
 #if defined(CSR_SPISDCARD_BASE) || defined(CSR_SDCORE_BASE)
-		sdcardboot();
+	sdcardboot();
 #endif
 #if defined(CSR_SATA_SECTOR2MEM_BASE)
-		sataboot();
+	sataboot();
 #endif
 #ifdef CSR_ETHMAC_BASE
 #ifdef CSR_ETHPHY_MODE_DETECTION_MODE_ADDR
-		eth_mode();
+	eth_mode();
 #endif
-		netboot();
+	netboot(0, NULL);
 #endif
-		printf("No boot medium found\n");
-	}
+	printf("No boot medium found\n");
 }
 
 int main(int i, char **c)
@@ -85,8 +87,11 @@ int main(int i, char **c)
 	irq_setmask(0);
 	irq_setie(1);
 #endif
+#ifdef CSR_UART_BASE
 	uart_init();
+#endif
 
+#ifndef CONFIG_SIM_DISABLE_BIOS_PROMPT
 	printf("\n");
 	printf("\e[1m        __   _ __      _  __\e[0m\n");
 	printf("\e[1m       / /  (_) /____ | |/_/\e[0m\n");
@@ -94,7 +99,7 @@ int main(int i, char **c)
 	printf("\e[1m     /____/_/\\__/\\__/_/|_|\e[0m\n");
 	printf("\e[1m   Build your hardware, easily!\e[0m\n");
 	printf("\n");
-	printf(" (c) Copyright 2012-2020 Enjoy-Digital\n");
+	printf(" (c) Copyright 2012-2021 Enjoy-Digital\n");
 	printf(" (c) Copyright 2007-2015 M-Labs\n");
 	printf("\n");
 #ifdef CONFIG_WITH_BUILD_TIME
@@ -136,6 +141,7 @@ int main(int i, char **c)
 #endif
 #endif
 	printf("\n");
+#endif // CONFIG_SIM_DISABLE_BIOS_PROMPT
 
         sdr_ok = 1;
 
@@ -159,6 +165,14 @@ int main(int i, char **c)
 	spiflash_init();
 #endif
 
+#ifdef CSR_VIDEO_FRAMEBUFFER_BASE
+	/* Initialize Video Framebuffer FIXME: Move */
+	video_framebuffer_vtg_enable_write(0);
+	video_framebuffer_dma_enable_write(0);
+	video_framebuffer_vtg_enable_write(1);
+	video_framebuffer_dma_enable_write(1);
+#endif
+
 	if(sdr_ok) {
 		printf("--============== \e[1mBoot\e[0m ==================--\n");
 		boot_sequence();
