@@ -13,27 +13,51 @@ import signal
 import os
 import time
 import serial
+import threading
 import multiprocessing
 import argparse
 import json
-import pty
-import telnetlib
+import socket
 
 # Console ------------------------------------------------------------------------------------------
 
 if sys.platform == "win32":
+    import ctypes
     import msvcrt
     class Console:
         def configure(self):
-            pass
+            # https://stackoverflow.com/a/36760881
+            # ENABLE_VIRTUAL_TERMINAL_PROCESSING
+            kernel32 = ctypes.windll.kernel32
+            kernel32.SetConsoleMode(kernel32.GetStdHandle(-11), 7)
 
         def unconfigure(self):
             pass
 
         def getkey(self):
             return msvcrt.getch()
+
+        # getch doesn't return Virtual Keycodes, but rather
+        # PS/2 Scan Codes. Keycodes starting with 0xE0 are
+        # worth handling.
+        def escape_char(self, b):
+            return b == b"\xe0"
+
+        def handle_escape(self, b):
+            return {
+                b"H" : b"\x1b[A",  # Up
+                b"P" : b"\x1b[B",  # Down
+                b"K" : b"\x1b[D",  # Left
+                b"M" : b"\x1b[C",  # Right
+                b"G" : b"\x1b[H",  # Home
+                b"O" : b"\x1b[F",  # End
+                b"R" : b"\x1b[2~", # Insert
+                b"S" : b"\x1b[3~", # Delete
+            }.get(b, None) # TODO: Handle ESC? Others?
+
 else:
     import termios
+    import pty
     class Console:
         def __init__(self):
             self.fd = sys.stdin.fileno()
@@ -52,13 +76,26 @@ else:
         def getkey(self):
             return os.read(self.fd, 1)
 
-# Crossover UART  ----------------------------------------------------------------------------------
+        def escape_char(self, b):
+            return False
+
+        def handle_escape(self, b):
+            return None
+
+# Bridge UART  -------------------------------------------------------------------------------------
 
 from litex import RemoteClient
 
-class CrossoverUART:
-    def __init__(self, host="localhost", base_address=0): # FIXME: add command line arguments
+class BridgeUART:
+    def __init__(self, name="uart_xover", host="localhost", base_address=0): # FIXME: add command line arguments
         self.bus = RemoteClient(host=host, base_address=base_address)
+        present = False
+        for k, v in self.bus.regs.d.items():
+            if f"{name}_" in k:
+                setattr(self, k.replace(f"{name}_", ""), v)
+                present = True
+        if not present:
+            raise ValueError(f"CrossoverUART {name} not present in design.")
 
     def open(self):
         self.bus.open()
@@ -76,63 +113,88 @@ class CrossoverUART:
     def pty2crossover(self):
         while True:
             r = os.read(self.file, 1)
-            self.bus.regs.uart_xover_rxtx.write(ord(r))
+            self.rxtx.write(ord(r))
 
     def crossover2pty(self):
         while True:
-            if self.bus.regs.uart_txfull.read():
+            if self.rxfull.read():
                 length = 16
-            elif not self.bus.regs.uart_xover_rxempty.read():
+            elif not self.rxempty.read():
                 length = 1
             else:
-                length = 0
-            if length:
-                r = self.bus.read(self.bus.regs.uart_xover_rxtx.addr, length=length, burst="fixed")
-                for v in r:
-                    os.write(self.file, bytes(chr(v).encode("utf-8")))
+                time.sleep(1e-3)
+                continue
+            r = self.bus.read(self.rxtx.addr, length=length, burst="fixed")
+            for v in r:
+                os.write(self.file, bytes(chr(v).encode("utf-8")))
 
 # JTAG UART ----------------------------------------------------------------------------------------
 
 from litex.build.openocd import OpenOCD
 
 class JTAGUART:
-    def __init__(self, config="openocd_xc7_ft2232.cfg", port=20000): # FIXME: add command line arguments
+    def __init__(self, config="openocd_xc7_ft2232.cfg", port=20000):
         self.config = config
         self.port   = port
 
     def open(self):
         self.file, self.name = pty.openpty()
-        self.jtag2telnet_thread = multiprocessing.Process(target=self.jtag2telnet)
-        self.jtag2telnet_thread.start()
+        self.jtag2tcp_thread = multiprocessing.Process(target=self.jtag2tcp)
+        self.jtag2tcp_thread.start()
         time.sleep(0.5)
-        self.pty2telnet_thread  = multiprocessing.Process(target=self.pty2telnet)
-        self.telnet2pty_thread  = multiprocessing.Process(target=self.telnet2pty)
-        self.telnet = telnetlib.Telnet("localhost", self.port)
-        self.pty2telnet_thread.start()
-        self.telnet2pty_thread.start()
+        self.pty2tcp_thread  = multiprocessing.Process(target=self.pty2tcp)
+        self.tcp2pty_thread  = multiprocessing.Process(target=self.tcp2pty)
+        self.tcp = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        self.tcp.connect(("localhost", self.port))
+        self.pty2tcp_thread.start()
+        self.tcp2pty_thread.start()
 
     def close(self):
-        self.jtag2telnet_thread.terminate()
-        self.pty2telnet_thread.terminate()
-        self.telnet2pty_thread.terminate()
+        self.jtag2tcp_thread.terminate()
+        self.pty2tcp_thread.terminate()
+        self.tcp2pty_thread.terminate()
 
-    def jtag2telnet(self):
+    def jtag2tcp(self):
         prog = OpenOCD(self.config)
         prog.stream(self.port)
 
-    def pty2telnet(self):
+    def pty2tcp(self):
         while True:
             r = os.read(self.file, 1)
-            self.telnet.write(r)
-            if r == bytes("\n".encode("utf-8")):
-                self.telnet.write("\r".encode("utf-8"))
-            self.telnet.write("\n".encode("utf-8"))
+            self.tcp.send(r)
 
-    def telnet2pty(self):
+    def tcp2pty(self):
         while True:
-            r = self.telnet.read_some()
+            r = self.tcp.recv(1)
             os.write(self.file, bytes(r))
 
+# Intel/Altera JTAG UART via nios2-terminal
+class Nios2Terminal():
+    def __init__(self):
+        from subprocess import Popen, PIPE
+        p = Popen("nios2-terminal", stdin=PIPE, stdout=PIPE)
+        self.p = p
+
+    def read(self):
+        return self.p.stdout.read(1)
+
+    def in_waiting(self):
+        # unfortunately p.stdout does not provide
+        # information about awaiting input
+        return False
+
+    def write(self, data):
+        if data is not None:
+            self.p.stdin.write(data)
+            try:
+                self.p.stdin.flush()
+            except BrokenPipeError:
+                print("nios2-terminal has terminated, exiting...\n")
+                sys.exit(1)
+
+    def close(self):
+        self.p.terminate()
+
 # SFL ----------------------------------------------------------------------------------------------
 
 sfl_prompt_req = b"F7:    boot from serial\n"
@@ -148,8 +210,6 @@ sfl_outstanding    = 128
 sfl_cmd_abort       = b"\x00"
 sfl_cmd_load        = b"\x01"
 sfl_cmd_jump        = b"\x02"
-sfl_cmd_flash       = b"\x04"
-sfl_cmd_reboot      = b"\x05"
 
 # Replies
 sfl_ack_success  = b"K"
@@ -220,7 +280,7 @@ def crc16(l):
 # LiteXTerm ----------------------------------------------------------------------------------------
 
 class LiteXTerm:
-    def __init__(self, serial_boot, kernel_image, kernel_address, json_images, flash):
+    def __init__(self, serial_boot, kernel_image, kernel_address, json_images):
         self.serial_boot = serial_boot
         assert not (kernel_image is not None and json_images is not None)
         self.mem_regions = {}
@@ -234,7 +294,6 @@ class LiteXTerm:
                 self.mem_regions[os.path.join(json_dir, k)] = v
             self.boot_address = self.mem_regions[list(self.mem_regions.keys())[-1]]
             f.close()
-        self.flash = flash
 
         self.reader_alive = False
         self.writer_alive = False
@@ -271,6 +330,8 @@ class LiteXTerm:
         sigint_time_current = time.time()
         # Exit term if 2 CTRL-C pressed in less than 0.5s.
         if (sigint_time_current - self.sigint_time_last < 0.5):
+            self.console.unconfigure()
+            self.close()
             sys.exit()
         else:
             self.sigint_time_last = sigint_time_current
@@ -293,7 +354,7 @@ class LiteXTerm:
     def receive_upload_response(self):
         reply = self.port.read()
         if reply == sfl_ack_success:
-            return
+            return True
         elif reply == sfl_ack_crcerror:
             print("[LXTERM] Upload to device failed due to data corruption (CRC error)")
         else:
@@ -306,8 +367,7 @@ class LiteXTerm:
         length = f.tell()
         f.seek(0, 0)
 
-        action = "Flashing" if self.flash else "Uploading"
-        print(f"[LXTERM] {action} {filename} to 0x{address:08x} ({length} bytes)...")
+        print(f"[LXTERM] Uploading {filename} to 0x{address:08x} ({length} bytes)...")
 
         # Prepare parameters
         current_address = address
@@ -326,12 +386,9 @@ class LiteXTerm:
             # Send frame if max outstanding not reached.
             if outstanding <= sfl_outstanding:
                 # Prepare frame.
-                frame = SFLFrame()
+                frame      = SFLFrame()
+                frame.cmd  = sfl_cmd_load
                 frame_data = f.read(min(remaining, self.payload_length-4))
-                if self.flash:
-                    frame.cmd = sfl_cmd_flash
-                else:
-                    frame.cmd = sfl_cmd_load
                 frame.payload = current_address.to_bytes(4, "big")
                 frame.payload += frame_data
 
@@ -347,10 +404,12 @@ class LiteXTerm:
                 # Inter-frame delay.
                 time.sleep(self.delay)
 
-            # Read response if availables.
+            # Read response if available.
             while self.port.in_waiting:
-                self.receive_upload_response()
-                outstanding -= 1
+                ack = self.receive_upload_response()
+                if ack:
+                    outstanding -= 1
+                    break
 
         # Get remaining responses.
         for _ in range(outstanding):
@@ -370,12 +429,6 @@ class LiteXTerm:
         frame.payload = int(self.boot_address, 16).to_bytes(4, "big")
         self.send_frame(frame)
 
-    def reboot(self):
-        print("[LXTERM] Rebooting the device.")
-        frame = SFLFrame()
-        frame.cmd = sfl_cmd_reboot
-        self.send_frame(frame)
-
     def detect_prompt(self, data):
         if len(data):
             self.prompt_detect_buffer = self.prompt_detect_buffer[1:] + data
@@ -400,12 +453,8 @@ class LiteXTerm:
             self.port.write(sfl_magic_ack)
         for filename, base in self.mem_regions.items():
             self.upload(filename, int(base, 16))
-        if self.flash:
-            # clear mem_regions to avoid re-flashing on next reboot(s)
-            self.mem_regions = {}
-        else:
-            self.boot()
-        print("[LXTERM] Done.");
+        self.boot()
+        print("[LXTERM] Done.")
 
     def reader(self):
         try:
@@ -426,12 +475,13 @@ class LiteXTerm:
 
     def start_reader(self):
         self.reader_alive = True
-        self.reader_thread = multiprocessing.Process(target=self.reader)
+        self.reader_thread = threading.Thread(target=self.reader)
+        self.reader_thread.setDaemon(True)
         self.reader_thread.start()
 
     def stop_reader(self):
         self.reader_alive = False
-        self.reader_thread.terminate()
+        self.reader_thread.join()
 
     def writer(self):
         try:
@@ -441,6 +491,10 @@ class LiteXTerm:
                     self.stop()
                 elif b == b"\n":
                     self.port.write(b"\x0a")
+                elif self.console.escape_char(b):
+                    b = self.console.getkey()
+                    ansi_seq = self.console.handle_escape(b)
+                    self.port.write(ansi_seq)
                 else:
                     self.port.write(b)
         except:
@@ -450,59 +504,73 @@ class LiteXTerm:
 
     def start_writer(self):
         self.writer_alive = True
-        self.writer_thread = multiprocessing.Process(target=self.writer)
+        self.writer_thread = threading.Thread(target=self.writer)
+        self.writer_thread.setDaemon(True)
         self.writer_thread.start()
 
     def stop_writer(self):
         self.writer_alive = False
-        self.writer_thread.terminate()
+        self.writer_thread.join()
 
     def start(self):
         self.start_reader()
         self.start_writer()
 
     def stop(self):
-        self.reader_thread.terminate()
-        self.writer_thread.terminate()
+        self.reader_alive = False
+        self.writer_alive = False
+
+    def join(self, writer_only=False):
+        self.writer_thread.join()
+        if not writer_only:
+            self.reader_thread.join()
 
 # Run ----------------------------------------------------------------------------------------------
 
 def _get_args():
     parser = argparse.ArgumentParser()
-    parser.add_argument("port",                                              help="Serial port")
-    parser.add_argument("--speed",       default=115200,                     help="Aerial baudrate")
+    parser.add_argument("port",                                              help="Serial port (eg /dev/tty*, bridge, jtag)")
+    parser.add_argument("--speed",       default=115200,                     help="Serial baudrate")
     parser.add_argument("--serial-boot", default=False, action='store_true', help="Automatically initiate serial boot")
     parser.add_argument("--kernel",      default=None,                       help="Kernel image")
-    parser.add_argument("--kernel-adr",  default="0x40000000",               help="Kernel address (or flash offset with --flash)")
+    parser.add_argument("--kernel-adr",  default="0x40000000",               help="Kernel address")
     parser.add_argument("--images",      default=None,                       help="JSON description of the images to load to memory")
-    parser.add_argument("--no-crc",      default=False, action='store_true', help="Disable CRC check (speedup serialboot)")
-    parser.add_argument("--flash",       default=False, action='store_true', help="Flash data with serialboot command")
+
+    parser.add_argument("--bridge-name", default="uart_xover",               help="Bridge UART name to use (present in design/csr.csv)")
+
+    parser.add_argument("--jtag-name",   default="jtag_uart",                help="JTAG UART type: jtag_uart (default), jtag_atlantic")
+    parser.add_argument("--jtag-config", default="openocd_xc7_ft2232.cfg",   help="OpenOCD JTAG configuration file for jtag_uart")
     return parser.parse_args()
 
 def main():
     args = _get_args()
-    if args.no_crc:
-        print("[LXTERM] --no-crc is deprecated and now does nothing (CRC checking is now fast)")
-    term = LiteXTerm(args.serial_boot, args.kernel, args.kernel_adr, args.images, args.flash)
+    term = LiteXTerm(args.serial_boot, args.kernel, args.kernel_adr, args.images)
 
-    bridge_cls = {"crossover": CrossoverUART, "jtag_uart": JTAGUART}.get(args.port, None)
-    if bridge_cls is not None:
-        bridge = bridge_cls()
+    if sys.platform == "win32":
+        if args.port in ["bridge", "jtag"]:
+            raise NotImplementedError
+    if args.port in ["bridge", "crossover"]: # FIXME: 2021-02-18, crossover for retro-compatibility remove and update targets?
+        bridge = BridgeUART(name=args.bridge_name)
         bridge.open()
         port = os.ttyname(bridge.name)
+    elif args.port in ["jtag"]:
+        if args.jtag_name == "jtag_atlantic":
+            term.port = Nios2Terminal()
+            port = args.port
+            term.payload_length = 128
+            term.delay          = 1e-6
+        elif args.jtag_name == "jtag_uart":
+            bridge = JTAGUART(config=args.jtag_config)
+            bridge.open()
+            port = os.ttyname(bridge.name)
+        else:
+            raise NotImplementedError
     else:
         port = args.port
     term.open(port, int(float(args.speed)))
     term.console.configure()
-    try:
-        term.start()
-        while True: pass
-    except:
-        if bridge_cls is not None:
-            bridge.close()
-        term.console.unconfigure()
-        term.stop()
-        term.close()
+    term.start()
+    term.join(True)
 
 if __name__ == "__main__":
     main()
